/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package controller;
import model.*;
import java.awt.Point;
import java.awt.event.*;
import java.util.Iterator;
import view.Application;

/**
 * <p>Title: Translator</p>
 * <p>Description: Class that performs a translation transformation on a Shape</p>
 * <p>Copyright: Copyright (c) 2003 Mohammed Elghaouat, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.4 $
 */
public class Translator extends AbstractTransformer{
  	private Point currentPoint;
  	private boolean bFirstTranslation;

	/**
	 * Default constructor
	 */
 	public Translator() {
		currentPoint = new Point();
    	        bFirstTranslation = true;
 	}

         /**
          *
          * @return Translator ID
          */
         public int getID(){
           return idTranslator;
         }

	/**
	 * Mouse dragged event management method
	 * @param e mouse event generated by the system
	 * @return transformation has been performed flag
	 */
 	protected boolean mouseDragged(MouseEvent e) {
    	if(!bFirstTranslation){
     		translate((int)(e.getPoint().getX()-currentPoint.getX()),
               		  (int)(e.getPoint().getY()-currentPoint.getY()));
    	} else {
      		bFirstTranslation = false;
    	}

    	currentPoint.setLocation(e.getPoint().getX(),e.getPoint().getY());
    	return true;
 	}

	/**
	 * Mouse released event management method
	 * @param e mouse event generated by the system
	 * @return transformation has been performed flag
	 */
 	protected boolean mouseReleased(MouseEvent e) {
  		bFirstTranslation = true;
  		return true;
 	}

	/**
	 * Set the current reference point where translation is to be applied
	 * @param aPt point coordinates values
	 */
 	public void setCurrentPoint(Point aPt) {
	    currentPoint.setLocation(aPt.getX(),aPt.getY());
 	}

	/**
	 * Perform the translation
	 * @param aX x displacement value
	 * @param aY y displacement value
	 */
 	private void translate(int aX,int aY) {
   		Shape shape;
   		Document doc = Application.getInstance().getActiveDocument();
   		Iterator iter = doc.getSelectedObjects().iterator();

   		while(iter.hasNext()){
     		shape = (Shape)iter.next();
                Point currentCenter = shape.getCenter();
                currentCenter.translate(aX,aY);
                shape.setCenter(currentCenter);
   		}
  	}
}