/*
 * Created on 27-Feb-04
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package model;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author paquette
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class Curve extends ShapeContainer implements ColoredShapeIF {

	/**
	 * @param centerX
	 * @param centerY
	 */
	public Curve(double centerX, double centerY) {
		super(centerX, centerY);
		boldStroke = new BasicStroke(3);
		resetCurrentControlPoints();
		recomputeLineSegments();
		color.setRed(64);
		color.setGreen(255);
		color.setBlue(64);
	}

	/**
	 * @param point
	 */
	public void addPoint(ControlPoint point) {
		addShape(point);
		recomputeLineSegments();
		notifyObservers();
	}

	/* (non-Javadoc)
	 * @see model.Shape#draw(java.awt.Graphics)
	 */
	public void draw(Graphics g) {
		drawCurve(g);
		super.draw(g);
	}

	/**
	 * @param g
	 */
	private void drawCurve(Graphics g) {
		g.setColor(getColor().toColor());
		Graphics2D g2d = (Graphics2D)g;
		Line2D.Double line = new Line2D.Double();
		Iterator segment = lineSegments.iterator();
		Point p0 = new Point(), p1;
		while (segment.hasNext()){
			List sections = ((List)segment.next());
			boolean bold = true;
			p1 = (Point)sections.get(0);
			drawLineStart(g, p1, (Point)sections.get(1));
			for (int i = 1; i < sections.size(); ++i){
				p0 = p1;
				p1 = (Point)sections.get(i);
				line.setLine(p0.x, p0.y, p1.x, p1.y);
				if (bold){
					Color initialColor = g.getColor();
					Stroke initialStroke = g2d.getStroke();
					g.setColor(Color.black);
					g2d.setStroke(boldStroke);
					g2d.draw(line);
					g2d.setStroke(initialStroke);
					g2d.setColor(initialColor);
				}
				g2d.draw(line);
				bold = !bold;
			}
			drawLineEnd(g, p0, p1);
		}
	}

	/**
	 * @param p0
	 * @param p1
	 */
	private void drawLineEnd(Graphics g, Point p0, Point p1) {
		double p01x = p1.x-p0.x;
		double p01y = p1.y-p0.y;
		double norm = Math.sqrt(p01x*p01x + p01y*p01y);
		double perpx = -p01y / norm * 6;
		double perpy = p01x / norm * 6;
		
		g.drawLine((int)(p1.x+perpx), (int)(p1.y+perpy), (int)(p1.x-perpx), (int)(p1.y-perpy));
	}

	/**
	 * @param p0
	 * @param point
	 */
	private void drawLineStart(Graphics g, Point p0, Point p1) {
		double p01x = p1.x-p0.x;
		double p01y = p1.y-p0.y;
		double norm = Math.sqrt(p01x*p01x + p01y*p01y);
		double perpx = -p01y / norm * 6;
		double perpy = p01x / norm * 6;
		
		g.drawLine((int)(p0.x+perpx), (int)(p0.y+perpy), (int)(p0.x-perpx), (int)(p0.y-perpy));
	}

	/* (non-Javadoc)
	 * @see model.Shape#isThisPointIn(java.awt.Point)
	 */
	public boolean isThisPointIn(Point aPt) {
		Iterator segment = lineSegments.iterator();
		Point p0, p1;
		while (segment.hasNext()){
			List sections = ((List)segment.next());
			p0 = (Point)sections.get(0);
			for (int i = 1; i < sections.size(); ++i){
				p1 = (Point)sections.get(i);
				double d = computePointToLineDistance(p0, p1, aPt);
				if (d <= SELECTION_THICKNESS)
					return true;
				p0 = p1;
			}
		}
		return false;
	}
	
	/**
	 * @return
	 */
	public int getNumberOfSections() {
		return numberOfSections;
	}

	/**
	 * @param i
	 */
	public void setNumberOfSections(int i) {
		numberOfSections = i;
		recomputeLineSegments();
		notifyObservers();
	}

    public void recomputeLineSegments() {
    	lineSegments.clear();
    	for (int i = 0; i < curveType.getNumberOfSegments(getShapes().size()); ++i){
    		lineSegments.add(computeLineSections(i));
    	}
    }
    
    public List computeLineSections(int segmentNumber){
    	List sections = new ArrayList(numberOfSections + 1);
    	double t = curveType.getMinT(segmentNumber);
    	double deltaT = (curveType.getMaxT(segmentNumber)-curveType.getMinT(segmentNumber)) /
    	                numberOfSections;
    	for (int i = 0; i < curveType.getNumberOfControlPointsPerSegment(); ++i){
    		currentControlPoints.set(i, curveType.getControlPoint(getShapes(), segmentNumber, i));
    	}
    	for (int i = 0; i < numberOfSections; ++i){
    		sections.add(curveType.evalCurveAt(currentControlPoints, t));
    		t += deltaT;
    	}
    	sections.add(curveType.evalCurveAt(currentControlPoints, curveType.getMaxT(segmentNumber)));
    	return sections;
    }
    
    public void update() {
    	recomputeLineSegments();
    	super.update();
    }
    
	/* (non-Javadoc)
	 * @see model.ColoredShapeIF#getColor()
	 */
	public Pixel getColor() {
		return color;
	}

	/* (non-Javadoc)
	 * @see model.ColoredShapeIF#setColor(model.Pixel)
	 */
	public void setColor(Pixel pixel) {
		color = pixel;
		notifyObservers();
	}
	
	public static double computePointToLineDistance(Point p0, Point p1, Point p) {
		double v01x = p1.x-p0.x;
		double v01y = p1.y-p0.y;
		double v0px = p.x-p0.x;
		double v0py = p.y-p0.y;
		double nv01_sq = v01x*v01x + v01y * v01y;
		double dot = (v01x*v0px + v01y*v0py) / nv01_sq;
		if (dot < 0) {
			return Math.sqrt(v0px*v0px + v0py * v0py);
		} else if (dot > 1) {
			double v1px = p.x-p1.x;
			double v1py = p.y-p1.y;
			return Math.sqrt(v1px*v1px + v1py * v1py);
		} else {
			double vpx = v0px - v01x * dot;
			double vpy = v0py - v01y * dot;  
			return Math.sqrt(vpx*vpx + vpy * vpy);
		}
	}
	
	public void setCurveType(CurveType ct) {
		curveType = ct;
		resetCurrentControlPoints();
		recomputeLineSegments();
		notifyObservers();
	}

	private void resetCurrentControlPoints() {
		currentControlPoints = new ArrayList(curveType.getNumberOfControlPointsPerSegment());
		ControlPoint cp = new ControlPoint(0,0);
		for (int i = 0; i <= curveType.getNumberOfControlPointsPerSegment(); ++i){
			currentControlPoints.add(cp);
		}
	}
	private int numberOfSections = 10;
	private CurveType curveType = new PolylineCurveType();
	private List lineSegments = new LinkedList(); 
	private ArrayList currentControlPoints;
	private Stroke boldStroke;
	private Pixel color = new Pixel();
}
