/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package model;
import java.awt.*;

/**
 * <p>Title: Disc</p>
 * <p>Description: Subclass of a Shape, representing a Disc.</p>
 * <p>Copyright: Copyright (c) 2003 Mohammed Elghaouat, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.2 $
 */
public class Disc extends ColoredShape {
  	private int radius;
        
  	/**
  	 * Default constructor.
  	 * @param aX x coordinate of the disc (center)
  	 * @param aY y coordinate of the disc (center)
  	 * @param aRadius radius of the disc
  	 */
  	public Disc(int centerX, int centerY, int radius) {
  		super(centerX, centerY);
    	this.radius = radius;
		getColor().setGreen(255);
  	}

	/**
	 * Disc rendering method.
	 * @param g Graphics rendering context
	 */
  	public void draw(Graphics g){
  		g.setColor(getColor().toColor());
   		g.fillOval((int)(getCenter().getX() - radius),
   				   (int)(getCenter().getY() - radius),
              	   radius*2,radius*2);
  	}

	/**
	 * Is the specified point in the disc area?
	 * @param aPt specified point coordinates
	 * @return point contained inside flag value
	 */
  	public boolean isThisPointIn(Point aPt){
    	return ((aPt.distance(getCenter()))<=radius);
  	}

	/**
	 * What is the radius of the disc?
	 * @return radius of the disc
	 */
  	public int getRadius(){
    	return radius;
  	}
}