/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package model;
import java.awt.*;
import java.util.*;

/**
 * <p>Title: Shape</p>
 * <p>Description: Abstract class that Shape objects implement.</p>
 * <p>Copyright: Copyright (c) 2004 Mohammed Elghaouat, Eric Paquette, Colin B-B</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.4 $
 */
public abstract class Shape implements ShapeObservable {
	public final int SELECTION_THICKNESS = 3;
	private java.util.List observers = new ArrayList();
	private Point center = new Point();

	/**
	 * Center initialized constructor.
	 * @param centerX Center's x coordinate
	 * @param centerY Center's y coordinate
	 */	
	public Shape(double centerX, double centerY) {
		setCenter(centerX, centerY);
	}
	
 	/**
 	 * Shape rendering method.
 	 * @param g Graphics rendering context
 	 */
 	public abstract void draw(Graphics g);
 
	/**
	 * Special Shape rendering method to draw the selection marker.
	 * Its default behaviour is to do nothing.
	 * @param g Graphics rendering context
	 */
	public void drawSelectionMarker(Graphics g){
	}
 
	/**
	 * Is the specified point in the shape area?
	 * @param aPt specified point coordinates
	 * @return point contained inside flag value
	 */
 	public abstract boolean isThisPointIn(Point aPt);
 
	/**
	 * Sets the center coordinate of the rectangle.
	 * @param aCenter center coordinate of the rectangle 
	 */ 	
 	public void setCenter(Point center) {
 		center.setLocation(center);
		notifyObservers(); 		
 	}

	/**
	 * Sets the center coordinate of the rectangle.
	 * @param aCenter center coordinate of the rectangle 
	 */ 	
	public void setCenter(double centerX, double centerY) {
		center.setLocation(centerX, centerY);
		notifyObservers(); 		
	}
 	
	/**
	 * What is the center coordinate of the disc?
	 * @return center coordinate of the disc
	 */   	
 	public Point getCenter() {
 		return center;
	}

	/**
	 * notify all observers when a new event happens
   	 */
	public void notifyObservers(){
		Iterator iter;
	 	iter = observers.iterator();
	 	
	 	while(iter.hasNext()){
	   		((ShapeObserver)iter.next()).update();
	 	}
    }

	/**
	 * add a new observer
	 */
	public void addObserver(ShapeObserver aObserver){
	   observers.add(aObserver);
	}
	
	/**
	 * remove an observer from the list of the observers
	 */
	public void removeObserver(ShapeObserver aObserver){
		 observers.remove(aObserver);
	}
	
	public void setContainer(ShapeContainer sc) {
		parent = sc;
	}
	
	public ShapeContainer getContainer() {
		return parent;
	}
	
	private ShapeContainer parent;
}