/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package view;

import java.awt.GridLayout;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.border.EtchedBorder;

import controller.FillColorsMediator;
import controller.ImageLineFiller;
import controller.RadioButtonsMediator;
import controller.SliderMediator;

/**
 * 
 * <p>Title: FillPanel</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004 Sbastien Bois, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version
 */
public class FillPanel extends JPanel 
{
	private JPanel thresholdSlidersPanel;
	private JPanel fillcolorRadioButtonPanel;
	private JPanel fillcolorButtonsPanel;
	
	private JSlider hTreshold;
	private JSlider sTreshold;
	private JSlider vTreshold;
	
	private JLabel  hTresholdValue;
	private JLabel  sTresholdValue;
	private JLabel  vTresholdValue;
	
	private JRadioButton rbBoundaryFill;
	private JRadioButton rbFloodFill;

	private Vector rbVector;
	private RadioButtonsMediator rbMediator;
	
	private JButton bFillColor;
	private JButton bBoundaryColor;

	public FillPanel(ImageLineFiller filler)
	{
		setLayout(new GridLayout(1,2));
		
		JPanel slidersPanel = new JPanel();
		slidersPanel.setLayout(new GridLayout(3,3));
		slidersPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		
		JPanel fillcolorRadioButtonPanel = new JPanel();
		fillcolorRadioButtonPanel.setLayout(new GridLayout(2,1));
		fillcolorRadioButtonPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		
		JPanel fillcolorButtonPanel = new JPanel();
		fillcolorButtonPanel.setLayout(new GridLayout(2,2));
		fillcolorButtonPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		
		JSlider hThreshold = new JSlider(JSlider.HORIZONTAL, 0, 180, 0);
		hThreshold.setMajorTickSpacing(60);
		hThreshold.setMinorTickSpacing(20);
		hThreshold.setPaintTicks(true);
		hThreshold.setPaintLabels(true);
				
		JSlider sThreshold = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
		sThreshold.setMajorTickSpacing(85);
		sThreshold.setMinorTickSpacing(28);
		sThreshold.setPaintTicks(true);
		sThreshold.setPaintLabels(true);
				
		JSlider vThreshold = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
		vThreshold.setMajorTickSpacing(85);
		vThreshold.setMinorTickSpacing(28);
		vThreshold.setPaintTicks(true);
		vThreshold.setPaintLabels(true);
		
		JLabel hThresholdValue = new JLabel();
		JLabel sThresholdValue = new JLabel();
		JLabel vThresholdValue = new JLabel();
		
		slidersPanel.add(new JLabel("Threshold H"));
		slidersPanel.add(hThreshold);
		slidersPanel.add(hThresholdValue);
		
		slidersPanel.add(new JLabel("Threshold S"));
		slidersPanel.add(sThreshold);
		slidersPanel.add(sThresholdValue);
		
		slidersPanel.add(new JLabel("Threshold V"));
		slidersPanel.add(vThreshold);
		slidersPanel.add(vThresholdValue);
		
		new SliderMediator(hThreshold, hThresholdValue, 
						   sThreshold, sThresholdValue, 
						   vThreshold, vThresholdValue,
						   filler);
		
		JRadioButton rbBoundaryFill 	= new JRadioButton("Boundary fill");
		fillcolorRadioButtonPanel.add(rbBoundaryFill);
		
		JRadioButton rbFloodFill    	= new JRadioButton("Flood fill");
		fillcolorRadioButtonPanel.add(rbFloodFill);
		
		RadioButtonsMediator rbMediator	= new RadioButtonsMediator(rbBoundaryFill, rbFloodFill, filler);
		
		bFillColor    = new JButton();
		bBoundaryColor = new JButton();
		
		FillColorsMediator fillColorsMediator = new FillColorsMediator(filler, bFillColor, bBoundaryColor);
		
		fillcolorButtonPanel.add(new JLabel("Fill color:"));
		fillcolorButtonPanel.add(bFillColor);

		slidersPanel.add(vThreshold);
		slidersPanel.add(vThresholdValue);
		
		fillcolorButtonPanel.add(new JLabel("Boundary color:"));
		fillcolorButtonPanel.add(bBoundaryColor);

		new SliderMediator(hThreshold, hThresholdValue, 
						   sThreshold, sThresholdValue, 
						   vThreshold, vThresholdValue,
						   filler);
		
		add(slidersPanel);
		add(fillcolorRadioButtonPanel);
		add(fillcolorButtonPanel);
	}
}
