/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package view;

import java.awt.CardLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;
import javax.swing.JTextField;

import controller.Coordinates;
import controller.FilteringTransformer;
import controller.TransformersIndex;

import model.KernelModel;

/**
 * 
 * <p>Title: KernelPanel</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003 Sbastien Bois, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version
 */
public class KernelPanel extends JPanel implements Observer{
	/**
	 * 
	 */
	private KernelModel _model;
	/**
	 * 
	 */
	private int _size;
	/**
	 * 
	 */
	private JPanel _contentPanel;
	/**
	 * 
	 */
	private EditBox[][] _kernelCells;
	/**
	 * 
	 */
	private FilteringTransformer transformer;
	
	/**
	 * 
	 * @param model
	 */
	public KernelPanel(TransformersIndex ti){
		setLayout(new CardLayout());
		
		_size  = 3;
		
		_kernelCells  = new EditBox[_size][_size];
		
		_contentPanel = new JPanel();
		_contentPanel.setLayout(new GridLayout(_size, _size)); 
		generateGrid();
		
		//
		add(_contentPanel, "");
		
		//
		transformer = ti.getTheFilter();//new FilteringTransformer();
	}
	/**
	 * 
	 *
	 */
	private void generateGrid(){
		for(int i = 0; i < _size; i++){
			for (int j = 0; j < _size; j++){
				// Get the value from the float array of the model
				
				// Add the edit box in this panel
				_kernelCells[i][j] =  new EditBox(i + 1, j + 1, 0.0f);
				_contentPanel.add(_kernelCells[i][j]);
			}
		}	
	}
	/**
	 * 
	 */
	public void update(Observable kernelModel, Object o) {
	}
	
	/**
	 * 
	 * <p>Title: EditBox</p>
	 * <p>Description: </p>
	 * <p>Copyright: Copyright (c) 2004 Sbastien Bois, Eric Paquette</p>
	 * <p>Company: (TS) - cole de Technologie Suprieure</p>
	 * @author unascribed
	 * @version
	 */
	class EditBox extends JTextField{
		/**
		 * 
		 */
		private Coordinates _coordinates;
		/**
		 * 
		 */
		private float _value;
		/**
		 * 
		 * @param row
		 * @param column
		 */
		public EditBox(int row, int column, float value){
			_coordinates = new Coordinates(row, column);
			_value = value;
			setText(Float.toString(value));
			addActionListener(new KernelCellChange(_model));
		}
		/**
		 * 
		 * @return
		 */
		public float getValue(){
			return _value;
		}
		/**
		 * 
		 *
		 */
		public void setValue(float v){
			_value = v;
		}
		/**
		 * 
		 * @return
		 */
		public Coordinates getCoordinates(){
			return _coordinates;
		}
		/**
		 * 
		 *
		 */
		public void updateTransformer () {
			// TODO EP Est-ce que _value est ncessairement  la bonne valeur?
			transformer.updateKernel(_coordinates, Float.parseFloat(getText()));
		}
		
		// controller
		/**
		 * 
		 * <p>Title: KernelCellChange</p>
		 * <p>Description: </p>
		 * <p>Copyright: Copyright (c) 2004 Sbastien Bois, Eric Paquette</p>
		 * <p>Company: (TS) - cole de Technologie Suprieure</p>
		 * @author unascribed
		 * @version
		 */
		class KernelCellChange implements ActionListener{
			KernelModel _model;
	  		
			 public KernelCellChange(KernelModel model){
				_model = model;
			 }
			 
			 public void actionPerformed(ActionEvent ae){
				//EditBox eb    = (EditBox)ae.getSource();
				//Coordinates c = eb.getCoordinates();
				//float value   = Float.parseFloat(eb.getText());
				//eb.setValue(Float.parseFloat(eb.getText()));
			  
			    // TODO EP Call the updateTransformerKernel method
				//_model.setKernelValue(c, value);
				
				//
				updateTransformerKernel();
				/*
				System.out.println("Row = "    + eb.getCoordinates().getRow());
				System.out.println("Column = " + eb.getCoordinates().getColumn());
				System.out.println("Value  = " + _model.getKernelValue(eb.getCoordinates()));
				*/
			 }
		 }
	}
	
	private void updateTransformerKernel() {
		for (int i = 0; i < _size; ++i) {
			for (int j = 0; j < _size; ++j) {
				// TODO EP Call the EditBox updateTransformer method
				_kernelCells[i][j].updateTransformer();
			}
		}
	}
}